# Generated by roxygen2: do not edit by hand

S3method(plot,pkgndep)
S3method(print,pkgndep)
export(co_heaviness)
export(current_bioc_version)
export(dependency_heatmap)
export(dependency_report)
export(heaviness)
export(heaviness_report)
export(load_pkg_db)
export(pkgndep)
export(pkgndep_opt)
export(reformat_db)
import(ComplexHeatmap)
import(GetoptLong)
import(GlobalOptions)
import(brew)
import(grDevices)
import(grid)
import(methods)
import(utils)
