

#' Load package database
#'
#' @param lib Local library. The value can be a character vector specifying the local library paths, or `NULL` which directly uses [`.libPaths()`]. A value of `NA` means not to use local libraries.
#' @param bioc_version Biconductor version.
#' @param verbose Whether to print messages.
#'
#' @details
#' It loads the package database from CRAN/Bioconductor and locally installed packages.
#'
#' The database object internaly is cached for repeated use of other functions in this package.
#'
#' @export
load_pkg_db = function(lib = .libPaths(), bioc_version = pkgndep_opt$bioc_version, verbose = TRUE) {

	ENV$pkg_db = prepare_db(lib = lib, bioc_version = bioc_version, verbose = verbose)
	ENV$bioc_version = bioc_version
	
}
