

ENV = new.env()

FIELDS = c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")

ENV$bioc_version = "not_configured"

#' Global parameters for pkgndep
#'
#' @param ... Arguments for the parameters, see "details" section
#' @param RESET Reset to default values.
#' @param READ.ONLY Please ignore.
#' @param LOCAL Pllease ignore.
#' @param ADD Please ignore.
#' 
#' @details
#' There are following parameters:
#' 
#' - `bioc_version` The bioconductor version. Use numeric version for devel branch.
#' 
#' @export
#' @import GlobalOptions
pkgndep_opt = setGlobalOptions(
    tmp_dir = list(.value = ".", .visible = FALSE),
    bioc_version = ENV$bioc_version
)


