% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkgndep.R
\name{pkgndep}
\alias{pkgndep}
\title{Package dependency analysis}
\usage{
pkgndep(
  package,
  lib = .libPaths(),
  bioc_version = pkgndep_opt$bioc_version,
  verbose = TRUE
)
}
\arguments{
\item{package}{Package name. The value can be 1. a CRAN/Bioconductor package, 2. an installed package, 3. a path of a local package, 4. URL of a GitHub repository.}

\item{lib}{Local library. The value can be a character vector specifying the local library paths, or \code{NULL} which directly uses \code{\link[=.libPaths]{.libPaths()}}. A value of \code{NA} means not to use local libraries.}

\item{bioc_version}{Bioconductor version.}

\item{verbose}{Whether to show messages.}
}
\value{
A \code{pkgndep} object.
}
\description{
Package dependency analysis
}
\examples{
\dontrun{
x = pkgndep("ComplexHeatmap")
}
# The `x` variable generated by `pkgndep()` is already saved in this package.
x = readRDS(system.file("extdata", "ComplexHeatmap_dep.rds", package = "pkgndep"))
x
dependency_heatmap(x)
}
