% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkgndep.R
\name{print.pkgndep}
\alias{print.pkgndep}
\title{Print method}
\usage{
\method{print}{pkgndep}(x, ...)
}
\arguments{
\item{x}{An object from \code{pkgndep}.}

\item{...}{Other arguments.}
}
\description{
Print method
}
