% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_data.R
\name{load_pkg_db}
\alias{load_pkg_db}
\title{Load package database}
\usage{
load_pkg_db(
  lib = .libPaths(),
  bioc_version = pkgndep_opt$bioc_version,
  verbose = TRUE
)
}
\arguments{
\item{lib}{Local library. The value can be a character vector specifying the local library paths, or \code{NULL} which directly uses \code{\link[=.libPaths]{.libPaths()}}. A value of \code{NA} means not to use local libraries.}

\item{bioc_version}{Biconductor version.}

\item{verbose}{Whether to print messages.}
}
\description{
Load package database
}
\details{
It loads the package database from CRAN/Bioconductor and locally installed packages.

The database object internaly is cached for repeated use of other functions in this package.
}
