% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global.R
\name{pkgndep_opt}
\alias{pkgndep_opt}
\title{Global parameters for pkgndep}
\usage{
pkgndep_opt(..., RESET = FALSE, READ.ONLY = NULL, LOCAL = FALSE, ADD = FALSE)
}
\arguments{
\item{...}{Arguments for the parameters, see "details" section}

\item{RESET}{Reset to default values.}

\item{READ.ONLY}{Please ignore.}

\item{LOCAL}{Pllease ignore.}

\item{ADD}{Please ignore.}
}
\description{
Global parameters for pkgndep
}
\details{
There are following parameters:
\itemize{
\item \code{bioc_version} The bioconductor version. Use numeric version for devel branch.
}
}
