% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{current_bioc_version}
\alias{current_bioc_version}
\title{Current Bioconductor Version}
\usage{
current_bioc_version()
}
\description{
Current Bioconductor Version
}
\details{
The version information is from https://bioconductor.org/config.yaml
}
