% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{heaviness_report}
\alias{heaviness_report}
\alias{dependency_report}
\title{HTML report for package dependency heaviness analysis}
\usage{
heaviness_report(pkg, file = NULL)

dependency_report(...)
}
\arguments{
\item{pkg}{An object from \code{\link[=pkgndep]{pkgndep()}}.}

\item{file}{The path of the html file. If it is not specified, the report will be automatically opened in the web browser.}

\item{...}{pass to \code{\link[=heaviness_report]{heaviness_report()}}.}
}
\value{
The path of the HTML file of the report.
}
\description{
HTML report for package dependency heaviness analysis
}
\examples{
if(interactive()) {
    x = readRDS(system.file("extdata", "ComplexHeatmap_dep.rds", package = "pkgndep"))
    heaviness_report(x)
}
}
